class_name DragonInstantiator

var _dragon_template: PackedScene
var _viewport: Viewport
var _window: Window

func _init(dragon: PackedScene, viewport: Viewport, window: Window) -> void:
	_window = window
	_viewport = viewport
	_dragon_template = dragon


func instantiate(properties: DragonProperties, drag: bool) -> Node:
	return _instantiate_dragon(properties, drag)


func _calculate_window_scale() -> Vector2:
	var viewport_size: Vector2 = _viewport.get_texture().get_size()
	var window_size: Vector2 = _window.get_size_with_decorations()
	
	return window_size / viewport_size


func _calculate_relative_position(position: Vector2) -> Vector2i:
	var scale: Vector2 = _calculate_window_scale()
	return Vector2i(Vector2(position) * scale)


func _instantiate_dragon(properties: DragonProperties, drag: bool) -> Node:
	var dragon: Dragon = _dragon_template.instantiate()
	dragon.properties = properties
	
	var window_position: Vector2i = DisplayServer.window_get_position()
	var window_size: Vector2i = DisplayServer.window_get_size()
	
	if drag == false:
		dragon.position = properties.position
	else:
		dragon.position = window_position + _calculate_relative_position(properties.position) - dragon.size / 2
	dragon.dress()

	dragon.main_window_rect = Rect2i(window_position, window_size)
	dragon.show()
	
	if drag:
		dragon.start_dragon_drag()
	
	return dragon
