extends Resource
class_name DragonOutfit


@export var allow_null: bool
@export var outfits: Array[SpriteFrames]
var _index: int = 0


func _init(outfits: Array[SpriteFrames] = []):
	self.outfits = outfits


func reset() -> void:
	_index = 0


func pick_next() -> SpriteFrames:
	_index += 1
	if allow_null:
		return _pick_nullable()
	else:
		return _pick_non_nullable()


func pick_previous() -> SpriteFrames:
	_index -= 1
	if allow_null:
		return _pick_nullable()
	else:
		return _pick_non_nullable()


func _pick_nullable() -> SpriteFrames:
	_index %= (len(outfits) + 1)
	if _index == 0:
		return null
	
	return outfits[_index - 1]


func _pick_non_nullable() -> SpriteFrames:
	_index %= len(outfits)
	return outfits[_index]


func get_texture(index: int) -> SpriteFrames:
	if index < 0 or index >= len(outfits):
		return null
	
	return outfits[index]


func get_current_index() -> int:
	if allow_null:
		return _index - 1
	else:
		return _index
