class_name SaveLoad

const save_location = "user://save.data"

var contents_to_save: Dictionary = {

								   }


func save() -> void:
	var file: FileAccess = FileAccess.open(save_location, FileAccess.WRITE)
	file.store_var(contents_to_save.duplicate(true))
	file.close()
	print('Saved game at: %s' % file.get_path_absolute())


func load() -> void:
	if FileAccess.file_exists(save_location):
		var file: FileAccess = FileAccess.open(save_location, FileAccess.READ)
		var data: Dictionary = file.get_var()
		file.close()

		contents_to_save = data.duplicate(true)
	else:
		contents_to_save.get_or_add('coins', 0)
		contents_to_save.get_or_add('dragons', [])
		contents_to_save.get_or_add('items', [])
		contents_to_save.get_or_add('tower_name', "")


func get_coins() -> int:
	return contents_to_save['coins']


func set_coins(coins: int) -> void:
	contents_to_save['coins'] = coins


func get_tower_name() -> String:
	return contents_to_save['tower_name']


func set_tower_name(name: String) -> void:
	contents_to_save['tower_name'] = name


func clear_dragons() -> void:
	contents_to_save['dragons'] = []


func add_dragon(dragon: DragonProperties) -> void:
	contents_to_save['dragons'].push_back({'name': dragon.name, 'origin': dragon.origin, 'color': dragon.color, 'hat': dragon.hat, 'shirt': dragon.shirt, 'decor': dragon.decor, 'pos': dragon.position})


func get_dragon(dragon: Dictionary) -> DragonProperties:
	var name: String       = dragon['name']
	var origin: String     = dragon['origin']
	var hat: int           = dragon['hat']
	var shirt: int         = dragon['shirt']
	var decor: int         = dragon['decor']
	var color: int         = dragon['color']
	var position: Vector2i = dragon['pos']
	var properties         = DragonProperties.new(name, origin, color, hat, shirt, decor)
	properties.position = position
	properties.id = randi()
	return properties


func get_dragons() -> Array[DragonProperties]:
	var dragons: Array[DragonProperties] = []
	for dragon in contents_to_save['dragons']:
		dragons.push_back(get_dragon(dragon))
	return dragons


func add_item(id: int, position: Vector2i) -> void:
	contents_to_save['items'].push_back({'id': id, 'position': position})


func get_items() -> Array:
	return contents_to_save['items']
